// include the library code:
#include <LiquidCrystal.h>
#include <Arduino.h> 
#include "lcd.hpp"
#include "logic.hpp"
/*
  LiquidCrystal Library - Hello World

 Demonstrates the use a 16x2 LCD display.  The LiquidCrystal
 library works with all LCD displays that are compatible with the
 Hitachi HD44780 driver. There are many of them out there, and you
 can usually tell them by the 16-pin interface.

 This sketch prints "Hello World!" to the LCD
 and shows the time.

  The circuit:
 * LCD RS pin to digital pin 12
 * LCD Enable pin to digital pin 11
 * LCD D4 pin to digital pin 5
 * LCD D5 pin to digital pin 4
 * LCD D6 pin to digital pin 3
 * LCD D7 pin to digital pin 2
 * LCD R/W pin to ground
 * LCD VSS pin to ground
 * LCD VCC pin to 5V
 * 10K resistor:
 * ends to +5V and ground
 * wiper to LCD VO pin (pin 3)

 Library originally added 18 Apr 2008
 by David A. Mellis
 library modified 5 Jul 2009
 by Limor Fried (http://www.ladyada.net)
 example added 9 Jul 2009
 by Tom Igoe
 modified 22 Nov 2010
 by Tom Igoe
 modified 7 Nov 2016
 by Arturo Guadalupi

 This example code is in the public domain.

 http://www.arduino.cc/en/Tutorial/LiquidCrystalHelloWorld

*/

// initialize the library by associating any needed LCD interface pin
// with the arduino pin number it is connected to
LiquidCrystal lcd(8, 9, 4, 5, 6, 7);

static const int lcdLightPin = 10;
static bool lcdLightOn = true;

void lcdSetup( void )
{
  pinMode(lcdLightPin, OUTPUT);

  lcdLigth( lcdLightOn );
  lcd.begin(16, 2);

  lcd.setCursor(0, 0);
  lcd.print( __DATE__ );
  lcd.setCursor(0, 1);
  lcd.print( __TIME__ );
  delay( 3000 );
  lcd.clear( );
}

void lcdTest(void) {
  // set the cursor to column 0, line 1
  // (note: line 1 is the second row, since counting begins with 0):
  lcd.setCursor(0, 0);
  // print the number of seconds since reset:
  lcd.print(lcdButtons());
}

int lcdButtons(void)
{
 int adc_key_in = analogRead(0);          // read analog A0 value
 // when read the 5 key values in the vicinity of the following：0,144,329,504,741
 // By setting different threshold, you can read the one button
 if (adc_key_in > 1000) return 0;//btnNONE;
 if (adc_key_in <  50)   return 16;
 if (adc_key_in < 300 /*250*/)  return 8;
 if (adc_key_in < 500 /*450*/)  return 4;
 if (adc_key_in < 700 /*650*/)  return 2;
 if (adc_key_in < 900 /*850*/)  return 1;

 // V1.0 Use the following version threshold：
/*
 if (adc_key_in < 50)   return btnRIGHT;
 if (adc_key_in < 195)  return btnUP;
 if (adc_key_in < 380)  return btnDOWN;
 if (adc_key_in < 555)  return btnLEFT;
 if (adc_key_in < 790)  return btnSELECT;
*/


 return 0;
}

extern
unsigned long ulStepLimit;

void lcdInfo( int status, int aclose, float tdif, float temp, int co, bool noiseLock, bool tempLock )
{
  static unsigned long Time = 0;

  if( Time + 300 >= millis () )
   return;
  char Buf[32];   
  Time = millis();
  lcd.setCursor(0, 0);
  Name( Buf, status );
  lcd.print(Buf);
  
  lcd.print(noiseLock?" S":" N");
  lcd.print(tempLock?"I ":"T ");

  long DTime = ulStepLimit - millis();
  if( DTime < 0 )
   DTime = 0;

  lcd.print(DTime/10000);
  lcd.print("   ");

  lcd.setCursor(0, 1);
  lcd.print( aclose );
  lcd.print( " " );  
  dtostrf( tdif, 4, 1, Buf );
  lcd.print( Buf );
  lcd.print( " " ); 
  dtostrf( temp, 4, 1, Buf );
  lcd.print( Buf );
  lcd.print( " " );  
  lcd.print( co/10 );
  lcd.print("   ");
}

bool keyUp( void )
{ return lcdButtons() == 8; }
bool keyDown( void )
{ return lcdButtons() == 4; }
bool keySelect( void )
{ return lcdButtons() == 1; }
bool keyLeft( void )
{ return lcdButtons() == 2; }
bool keyRight( void )
{ return lcdButtons() == 16; }

void lcdLigth( bool on )
{ digitalWrite(lcdLightPin, on?HIGH:LOW); }

void lcdLightSwap( void )
{
  static unsigned long ulLast = 0;
  if( ulLast + 3000 >= millis() )
   return;
  ulLast = millis(); 
  lcdLightOn = lcdLightOn?LOW:HIGH;
  lcdLigth( lcdLightOn );
}
