#include <Arduino.h>
#include "logic.hpp"
#include "motor.hpp"
#include "noise.hpp"
#include "temp.hpp"
#include "oxygen.hpp"

unsigned long ulStepStart,
              ulStepLimit;

static int iMode = 0;

int timeSet( int state, int aspect = 0 )
{
  unsigned long blocks;
  switch( state )
  {
    case stNCWM:
    case stNCWA: blocks = toNoise; break;
    case stVOWM: 
    case stVOWA: blocks = toVent; break;
    case stMOW:
    case stMCW: blocks = toManual; break;
    default: blocks = 0;
  }

  if( blocks == 0 )
  switch( aspect )
  {
    case stNoise: blocks = toNoise; break;
  }
  
  ulStepStart = millis();
  ulStepLimit = ulStepStart + blocks*1000;
  return state;
}

bool timeEnd()
{ Serial.print("time:");
  Serial.print(millis());
  Serial.print('/');
  Serial.println(ulStepLimit);
  return ulStepLimit <= millis();
}

bool timeMove()
{ return millis() <= ulStepStart + (unsigned long)toMove*1000; }
/*
int Parent( int state )
{
 int Parent = 0;
 
 if( state & stParentAuto )
  Parent |= stAuto;
 else
 if( state & stParentManual )
  Parent |= stManual;
  
 if( state & stNoise )
  Parent |= stOpen;
 else
 if( state & stVent )
  Parent |= stClose;

 return Parent;
}*/

int Step( int state )
{ 
  switch( state )
  {
    case stStart:
      state = timeSet( stAOW );
     break;
    case stAOW:
      if( noiseIsOne( ) ) /* may be periodicaly open-close-open */
       state = timeSet( stNCWA );
      else
      if( keyDown() )
       state = timeSet( stMCW );
      else
      if( tempToClose() )
       state = timeSet( stACW );
     break;
    case stACW:
      if( needVent( ) && timeEnd() )
       state = timeSet( stVOWA );
      else
      if( keyUp() )
       state = timeSet( stMOW );
      else
      if( tempToOpen() && timeEnd() )
       state = timeSet( stAOW );
     break;
    case stNCWM:
    case stNCWA:
      if( timeEnd() )
      {
       if( !noiseIs( ) )
        // we dont know ho open like manual value//state = timeSet( Parent( state ) );
        state = timeSet( stAOW );
       else
        state = timeSet( stACW, stNoise );
      }
      else
      if( keyUp( ) )
       state = timeSet( stMOW );
     break;
    case stMOW:
      if( keySelect() )
       state = timeSet( stAOW );
      else
      if( keyDown() )
       state = timeSet( stMCW );
      else
      if( noiseIsOne( ) && timeEnd() )
       state = timeSet( stNCWM );
     break;
    case stMCW:
      if( keySelect() )
       state = timeSet( stACW );
      else
      if( keyUp() )
       state = timeSet( stMOW );
      else
      if( needVent() && timeEnd() )
       state = timeSet( stVOWM );
     break;
    case stVOWM:
    case stVOWA:
      if( noiseIsOne( ) && timeEnd() )
      {
       oxClear( );
       state = timeSet( stACW/*dont know how close like manual // Parent( state )*/, stNoise );
      }
      else
      if( keyDown() )
      {
       oxClear( );
       state = timeSet( stMCW );
      }
      else
      if( timeEnd() )
      {
       oxClear( );
       state = timeSet( stACW/*dont know how close like manual // Parent( state )*/ );
      }
     break;
  }
  return state; 
}

int Jumps( int state )
{
  int State = state;
  do
  {
    state = State;
    State = Step( state );
  }while( state != State ); /* infinity loop may be */

  return State;
}

int iAutoClose = 0;

int Work( int state )
{
 int State = Jumps( state );
 if( State != state )
 {
  if( (State & stOpen) && !(state & stOpen) )
  {
   if( State & stAuto )
    motorOpenSlo();
  }
  else
  if( (State & stClose) && !(state & stClose) )
  {
   if( State & stAuto ) 
   { motorCloseSlo();
     iAutoClose ++;
   }
  }

  if( (State & stOpen) != (state & stOpen) )
   tempDiff( State & stOpen );
 }
 else
 switch( State )
  {
    case stMOW: if( keyUp( ) ) { motorOpen(); delay( 300 ); if( !keyUp( ) ) motorStop(); }
     break;
    case stMCW: if( keyDown( ) ) { motorClose(); delay( 300 ); if( !keyDown( ) ) motorStop(); }
     break;
    default:
                if( !timeMove() )
                 motorStop();
  }

 if( keyLeft( ) )
  lcdLightSwap();

 if( keyRight( ) )
 {
  static unsigned long Keyp = 0;
  if( Keyp + 300 < millis() )
  {
   Keyp = millis();
   iMode = (iMode+1)%3;
   noiseLock = (iMode != 0);
   tempLock = (iMode == 2);
  }
 }

 oxStore( State );

 return State;
}

void Name( char * name, int state )
{
  int i = 0;
  if( state & stAuto )
   name[i++] = 'A';
  else
  if( state & stManual )
   name[i++] = 'M';
  else
   name[i++] = '_';
   
  if( state & stOpen )
   name[i++] = 'O';
  else
  if( state & stClose )
   name[i++] = 'C';
  else
   name[i++] = '_';

  if( state & stNoise )
   name[i++] = 'N';
  else
  if( state & stVent )
   name[i++] = 'V';
  else
   name[i++] = '_';

  if( state & stParentAuto )
   name[i++] = 'a';
  else
  if( state & stParentManual )
   name[i++] = 'm';
  else
   name[i++] = '_';

  name[i]='\0';
}
