#include <OneWire.h>
#include "temp.hpp"

// OneWire DS18S20, DS18B20, DS1822 Temperature Example
//
// http://www.pjrc.com/teensy/td_libs_OneWire.html
//
// The DallasTemperature library can do all this work for you!
// https://github.com/milesburton/Arduino-Temperature-Control-Library

static OneWire  ds(2);  // on pin 10 (a 4.7K resistor is necessary)
static byte addr[8]={0};

bool tempSearch(void)
{
  byte i;
  if ( !ds.search(addr)) {
    Serial.println("No more addresses.");
    Serial.println();
    ds.reset_search();
    //delay(250);
    return false;
  }
  
  Serial.print("ROM =");
  for( i = 0; i < 8; i++) {
    Serial.write(' ');
    Serial.print(addr[i], HEX);
  }

  if (OneWire::crc8(addr, 7) != addr[7]) {
      Serial.println("CRC is not valid!");
      return false;
  }
  Serial.println();
  return true;
}

void tempSetup(void)
{
  tempSearch();
  tempGet();
}

byte tempType()
{
  byte type_s;
    // the first ROM byte indicates which chip
  switch (addr[0]) {
    case 0x10:
      //Serial.println("  Chip = DS18S20");  // or old DS1820
      type_s = 1;
      break;
    case 0x28:
      //Serial.println("  Chip = DS18B20");
      type_s = 0;
      break;
    case 0x22:
      //Serial.println("  Chip = DS1822");
      type_s = 0;
      break;
    default:
      //Serial.println("Device is not a DS18x20 family device.");
      type_s = 0xFF;
  } 
  return type_s;
}

void tempBegin(void)
{
  ds.reset();
  ds.select(addr);
  ds.write(0x44, 1);        // start conversion, with parasite power on at the end
}

float celsius = -300;

float tempGet(void) {
  static unsigned long ulLast = -1, ulPause = 1000;
  static int16_t raw;
  byte i;
  byte present = 0;
  byte type_s = tempType();
  byte data[12];
  
  if( ulLast == -1 )
  {
   tempBegin();
   ulLast = millis();
   delay(ulPause + 10);     // maybe 750ms is enough, maybe not
   // we might do a ds.depower() here, but the reset will take care of it. 
  }

  if( ulLast + ulPause < millis() )
  {
  ulPause = 60000;
    
  present = ds.reset();
  ds.select(addr);    
  ds.write(0xBE);         // Read Scratchpad

  Serial.print("  Data = ");
  Serial.print(present, HEX);
  Serial.print(" ");
  for ( i = 0; i < 9; i++) {           // we need 9 bytes
    data[i] = ds.read();
    Serial.print(data[i], HEX);
    Serial.print(" ");
  }
  Serial.print(" CRC=");
  Serial.print(OneWire::crc8(data, 8), HEX);
  Serial.println();

  // Convert the data to actual temperature
  // because the result is a 16 bit signed integer, it should
  // be stored to an "int16_t" type, which is always 16 bits
  // even when compiled on a 32 bit processor.
  raw = (data[1] << 8) | data[0];
  if (type_s) {
    raw = raw << 3; // 9 bit resolution default
    if (data[7] == 0x10) {
      // "count remain" gives full 12 bit resolution
      raw = (raw & 0xFFF0) + 12 - data[6];
    }
  } else {
    byte cfg = (data[4] & 0x60);
    // at lower res, the low bits are undefined, so let's zero them
    if (cfg == 0x00) raw = raw & ~7;  // 9 bit resolution, 93.75 ms
    else if (cfg == 0x20) raw = raw & ~3; // 10 bit res, 187.5 ms
    else if (cfg == 0x40) raw = raw & ~1; // 11 bit res, 375 ms
    //// default is 12 bit resolution, 750 ms conversion time
  }
   tempBegin();
   ulLast = millis();
  }

  if( type_s == 0xFF )
   celsius = 24;
  else
   celsius = (float)raw / 16.0;
  
  //Serial.print("  Temperature = ");
  //Serial.print(celsius);
  //Serial.println(" Celsius");

  return celsius;
}

bool tempToCloseLim(void)
{ return (celsius < 17) || (celsius > 30); }

bool tempToCloseDelta(boolean cl)
{ 
  float d = cl ? 1 : -1;
  return ((celsius > 25 + d)&&(tempDif >= 0)) || ((celsius < 20 - d)&&(tempDif <= 0)); }

bool tempLock = false;

bool tempToOpen(void)
{ return !tempLock && (!tempToCloseDelta( false )&&!tempToCloseLim()); }

bool tempToClose(void)
{ return !tempLock && (tempToCloseDelta( true ) || tempToCloseLim()); }


float tempDif = 0; /* -1 - weather cold; 1 - weather hot */
void tempDiff( bool isopen )
{
 static unsigned long Time = 0;
 static float Temp = celsius;

 if( Temp > -300 &&
     Time + 2*60000 <= millis()
   )
 {
  tempDif = Temp - celsius;
  if( !isopen )
   tempDif = - tempDif;
 }
 
 Temp = celsius;
 Time = millis();
}
